//version 4.0.2

//layout toc title(subsection) link(subsection) 
//layout toc secfont(bold) subsubsecfont(italic) 
//toctitle mkproject and boilerplate: Automate the beginning

/*toctxt
{center:Maarten L. Buis}
{center:University of Konstanz}

toctxt*/


//slide ------------------------------------------------------------------------
//title Starting is repetitive and boring

/*txt 
{pstd}
When using Stata to do research it is good to do so in a very systematic and 
standardized way 

{pmore}
Use commands like {help clear}, {help log}, and {help version} at the beginning 
of the .do file, and {cmd:log close} at the end

{pmore}
Use standardized filenames for your .do files, log files and data files.

{pmore}
Use a standardized set of sub-folders

{pstd}
This helps you if you need to get back to that project after some time 

{pstd}
This helps others when they want to replicate your study

{pstd}
Doing standardized stuff is boring, but it also opens the possibility of 
automation

{pstd}
This is the problem {cmd:mkproject} and {cmd:boilerplate} intend to solve
txt*/
//endslide ---------------------------------------------------------------------
//file dependencies.do
//file cleandirs.ado

//slide ------------------------------------------------------------------------
//title Example of basic use

/*txt
{pstd}
Lets assume we want to start a new research project on the development of 
inequality in cultural capital over time.

{pstd}
We think of a convenient "abbreviation", like {it:ccineq}

{pstd}
We also need to choose a template, in this case I chose a template based on 
/*cite Long2009 */
txt*/

//ex
mkproject ccineq, template(longt)
//endex

/*txt
{pstd}
Especially for preparing data I usually require multiple .do files 

{pstd}
To add a .do datapreparation .do file I type:
txt*/

//ex
boilerplate ccineq_dta02, template(dta)
//endex

/*txt
{pstd}
How do I know what templates are available?
txt*/

//ex
mkproject, query
boilerplate, query
//endex

//endslide ---------------------------------------------------------------------

//anc --------------------------------------------------------------------------
//title default templates

/*txt
{pstd}
The {cmd:template()} option is optional. If you don't specify it, {cmd:boilerplate}
and {cmd:mkproject} will use the default templates. 

{pstd}
The defaults are marked with a star if you use the {cmd:query} option
txt*/

//ex
mkproject, query
boilerplate, query
//endex

/*txt
{pstd}
You can change the default with the {cmd:default()} option
txt*/

//ex
boilerplate, default(ana)
boilerplate, query
boilerplate, resetdefault
boilerplate, query
//endex
//endanc -----------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Creating your own templates

/*txt 
{pstd}
On October 20th 2023 Chris Stefancik asked on Statalist about the possibility of
starting a .do file with boilerplate code

{pstd}
He also gave an example of such boilerplate code he wanted to start with for his
.do files: 

//codefile chris_stefancik.do boilerplate

{pstd}
As a minimum template we can just add {cmd:<body>} at the begin of that file and
{cmd:</body>} at the end

//codefile cs1.do cs1.do
txt*/ 

//ex
boilerplate, create(cs1.do) replace
boilerplate, query
//endex

//ex
boilerplate ccineq_dta01.do, template(cs1)
//endex
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title Header

/*txt 
{pstd}
When we look at {cmd:boilerplate, query} we see the {cmd:cs1} template, but there
is no label telling us what that template is supposed to be for.

{pstd}
When we look at the help-file for the {cmd:cs1} template, there is no description
telling us how that template is intended to be used.

{pstd}
This is meta data that Stata can't create for us.

{pstd}
Meta data is stored in the header, indicated by the {cmd:<header>} and 
{cmd:</header>} tags

{pstd}
It can contain five types of meta data:

{pmore} 
        {cmd:<mkproject>} Indicates that this is a template, and the word after 
        it can be either {it:project} or {it:boilerplate}, to indicate whether 
		this is a {cmd:mkproject} or {cmd:boilerplate} template. You can specify
        it in your text file or if you don't specify it, {cmd:boilerplate, create()}
		or {cmd:mkproject, create()} will add it for you.

{pmore}
        {cmd:<version>} Indicates which version of {cmd:boilerplate} is used to 
		create this template. {cmd:boilerplate} uses that information to ensure 
		backwards compatability. You can specify it yourself, or if you don't 
		specify it, {cmd:boilerplate, create()} will set it to the current 
		version of {cmd:boilerplate}.

{pmore}		
        {cmd:<label>} gives the short description of your template, that will be 
		shown with {cmd:boilerplate, query}

{pmore}
        {cmd:<reqs>} specifies a community contributed package that is required 
		for this template.

{pmore}		
        The lines between {cmd:<description>} and {cmd:</description>} give a 
		longer description of the template as it will appear in the help-file. 
		It may contain smcl tags.
		
{pstd}
We can improve Chris' template like so:

//codefile cs2.do cs2.do
txt*/

//ex
boilerplate, create(cs2.do)
boilerplate, query
//endex

/*txt
{pstd}
When we create a template, it will remain on your computer

{pstd}
So you don't have to do the {cmd:boilerplate, create()} step every time you want
to use your template

{pstd}
But if you no longer want it, you can remove it with {cmd:boilerplate, remove()}
txt*/

//ex
boilerplate, remove(cs1)
//endex
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title body

/*txt
{pstd}
Chris' file contains a change log with a date, and we know what that date should 
be: the date when you run {cmd:boilerplate}

{pstd}
There is a tag you can use that will be replaced with the current date: {cmd:<date>}

{pstd}
That same date tag can also be used to fill in a part of the first comment block

{pstd}
That comment block also asked for the file name, which is available with the {cmd:<fn>} tag

{pstd}
It would be easier to find a log file of interest if it contains the filename of
of the .do file that created it in addition to the date and time. The {cmd:<stub>}
tag will be replaced by the filename without the extension.

{pstd}
So we can improve our template:

//codefile cs3.do cs3.do
txt*/

//ex
boilerplate, create(cs3.do)
boilerplate ccineq_dta01.do, template(cs3)
//endex

/*txt
{pstd}There are 6 tags you could use:{p_end}
{pmore}{cmd:<stata_version>} will be replaced by the Stata version{p_end}
{pmore}{cmd:<date>} will be replaced by the date{p_end}
{pmore}{cmd:<fn>} will be replaced by the file name{p_end}
{pmore}{cmd:<stub>} will be replaced by the file name without the suffix{p_end}
{pmore}{cmd:<abbrev>} will be replaced by the file name without the suffix up to the last underscore{p_end}
{pmore}{cmd:<basedir>} will be replaced by the directory in which the file is saved{p_end}
{pmore}{cmd:<as of #>} will include whatever comes after that tag only if the Stata version is # or higher{p_end}

{pstd}So in the example about:{p_end}
{pmore}{cmd:<fn>} is ccineq_dta01.do{p_end}
{pmore}{cmd:<stub>} is ccineq_dta01{p_end}
{pmore}{cmd:<abbrev>} is ccineq{p_end}
txt*/
//endslide ---------------------------------------------------------------------

//slide ------------------------------------------------------------------------
//title new project

/*txt
{pstd}
Say Chris wants to start a new project called {it:ccineq}, he wants to take the following steps{p_end}

{pmore}1. Create a folder with the following structure:{p_end}


            ccineq \
            ├── analysis \
            │   └── stata logs \
            └── text \

{pmore}2. He wants to use {cmd:boilerplate} to create the following files:{p_end}
{p 12 12 2}- {it:ccineq01.do} within the analysis folder with the {cmd:cs3} template {p_end}
{p 12 12 2}- {it:researchlog.md} within the root folder with the {cmd:rlog} template {p_end}

{pmore}3. After creating those files he wants to run the following commands{p_end}
{p 12 12 2}{cmd:dirtree}{p_end}
{p 12 12 2}{cmd:qui cd analysis}{p_end}
{p 12 12 2}{cmd:doedit ccineq01.do}{p_end}

{pstd}
He can create his own template for {cmd:mkproject}

{pstd}
The header works pretty much the same way as in {cmd:boilerplate}, except you 
would use {cmd:<mkproject> project} to indicate that it is a template for 
{cmd:mkproject}

{pstd}Within the body you can use{p_end} 
{pmore}the {cmd:<dir>} tag to specify sub-directories{p_end}
{pmore}the {cmd:<file>} tag to specify files to created with {cmd:boilerplate}{p_end}
{pmore}the {cmd:<cmd>} tag to specify commands that are to be run{p_end}

{pstd}
Here is a template that does that:

//codefile pcs.do pcs.do
txt*/

//ex
mkproject, create(pcs.do) replace
mkproject, query
mkproject ccineq, template(pcs)
//endex
//endslide ---------------------------------------------------------------------

//bib
//title bibliography
/*bib
@Book{Long2009,
  author    = {J. Scott Long},
  publisher = {Stata Press},
  title     = {The Workflow of Data Analysis Using Stata: Principles and practice for effective data management and analysis},
  year      = {2009},
  address   = {College Station, TX},
}

bib*/
//endbib
